"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var redux_1 = require("redux");
var create_action_1 = require("./create-action");
test('creates actions with constant payload creator', function () {
    var ac = create_action_1.default('foo1', function () { return 'bar'; });
    expect(ac.type).toBe('foo1');
    expect(ac()).toEqual({
        type: 'foo1',
        payload: 'bar',
    });
});
test('creates actions with untyped identity payload creator', function () {
    var ac = create_action_1.default('foo2');
    expect(ac.type).toBe('foo2');
    expect(ac('bar')).toEqual({
        type: 'foo2',
        payload: 'bar',
    });
});
test('creates actions with void identity payload creator', function () {
    var ac = create_action_1.default('void-action');
    expect(ac.type).toBe('void-action');
    expect(ac()).toEqual({
        type: 'void-action',
    });
});
test('creates actions with void identity payload creator', function () {
    var ac = create_action_1.default('void-action-untyped');
    expect(ac.type).toBe('void-action-untyped');
    expect(ac()).toEqual({
        type: 'void-action-untyped',
    });
});
test('creates actions with explicit payload creator', function () {
    var ac = create_action_1.default('foo3', function (num) { return "" + num; });
    expect(ac.type).toBe('foo3');
    expect(ac(4)).toEqual({
        type: 'foo3',
        payload: '4',
    });
});
test('creates actions with explicit payload creator (many args)', function () {
    var ac = create_action_1.default('foo4', function (num) { return "" + num * 2; });
    expect(ac.type).toBe('foo4');
    expect(ac(4, true, 'bar')).toEqual({
        type: 'foo4',
        payload: '8',
    });
});
test('meta customizer', function () {
    var ac = create_action_1.default('with-meta', function (s) { return s; }, function (s) { return s.toUpperCase(); });
    expect(ac.type).toBe('with-meta');
    expect(ac('foo')).toEqual({
        type: 'with-meta',
        payload: 'foo',
        meta: 'FOO',
    });
});
test('error payload', function () {
    var ac = create_action_1.default('with-error');
    expect(ac.type).toBe('with-error');
    expect(ac(new Error('Rabble!'))).toEqual({
        type: 'with-error',
        payload: new Error('Rabble!'),
        error: true,
    });
});
test('polymorphic payload', function () {
    var ac = create_action_1.default('with-polymorphic');
    expect(ac.type).toBe('with-polymorphic');
    expect(ac('foo')).toEqual({
        type: 'with-polymorphic',
        payload: 'foo',
    });
    expect(ac(new Error('Rabble!'))).toEqual({
        type: 'with-polymorphic',
        payload: new Error('Rabble!'),
        error: true,
    });
});
test('works with bindActionCreators from redux', function () {
    var ac1 = create_action_1.default('foo5', function () { return 'baz'; });
    expect(ac1.type).toBe('foo5');
    var ac2 = create_action_1.default('bar5', function (s) { return s; });
    expect(ac2.type).toBe('bar5');
    var acs = redux_1.bindActionCreators({
        ac1: ac1,
        ac2: ac2,
    }, function (action) {
        expect(action.type).toBe('foo5');
        return action;
    });
    acs.ac1();
});
test('toString', function () {
    var ac = create_action_1.default('foo6');
    expect(ac.type).toBe('foo6');
    expect(ac.toString()).toBe('foo6');
});
//# sourceMappingURL=create-action.test.js.map